#include <boost/python/class_builder.hpp>

#include <string>

namespace {

  class base
  {
    public:
      base(const std::string& major) : m_major(major) {}
      const std::string& get_major() const { return m_major; }
      std::string operator()() const { return m_major + " through operator()"; }
    private:
      std::string m_major;
  };

  class derived : public base
  {
    public:
      derived(const std::string& major, const std::string& minor)
        : base(major), m_minor(minor) {}
      const std::string& get_minor() const { return m_minor; }
    private:
      std::string m_minor;
  };
}

BOOST_PYTHON_MODULE_INIT(declare_base)
{
  try
  {
    boost::python::module_builder this_module("declare_base");

    boost::python::class_builder<base> py_base(this_module, "base");
    boost::python::class_builder<derived> py_derived(this_module, "derived");

    py_derived.declare_base(py_base, boost::python::without_downcast);

    py_base.def(boost::python::constructor<const std::string&>());
    py_base.def(&base::get_major, "get_major");
    py_base.def(&base::operator(), "__call__");
    py_base.def(&base::operator(), "explicit_call");

    py_derived.def(boost::python::constructor<const std::string&, const std::string&>());
    py_derived.def(&derived::get_minor, "get_minor");
  }
  catch(...)
  {
    boost::python::handle_exception();
  }
}
